function [f, df] = piston2c(x, t)
% Piston simulator, 2D coarse model
% Calculate the coarse model response for a given set of parameters
%
% [f, df] = piston2c(x, t)

% js@imm.dtu.dk, September 12, 2002

if nargout>1
   [f, df] = fdgrad(@piston, x, [], 0, t.state);
   if length(f) == 1, df = df'; end
else
   f = piston(x, t.state);
end

if ~isempty(t.target)
  f = f - t.target;
  f = [f; -f];
  if nargout > 1
      df = [df; -df];
  end
end
