function [f, df] = piston2f(x, t)
% Piston simulator, 2D fine model
% Calculate the fine model response for a given set of parameters
%
% [f, df] = piston2f(x, t)

% js@imm.dtu.dk, September 12, 2002

if nargout>1
  [f, df] = fdgrad(@piston2fwrap, x, [], 0, t.state);
  if length(f) == 1, df = df'; end
else
   f = piston2fwrap(x, t.state);
end

if ~isempty(t.target)
  f = f - t.target;
  f = [f; -f];
  if nargout > 1
    df = [df; -df];
  end
end

function res = piston2fwrap(x, t)
res = piston([200 x(1) 175 x(2) 100 200]', t);
