function [s, ds] = combined_corrected_model(x, coarse, xk, zk, fk, ck, g, B, D, w, varargin)
% Combined corrected model
%
%   s_k(x) = w ( g .* (c(p_k(x)) - c(z_k)) + f(x_k) ) + (1 - w) l_k(x)
%
% where c(p_k(x)) is a 'mapped_model'
% and l_k(x) = D_k * (x - x_k) + f(x_k) is a
% a linear approximation to f
%
% [function used in space mapping algorithms]

% js@imm.dtu.dk, January 19, 2003

w1 = 1 - w;

if w > 0.0
  [cp, dcp] = mapped_model(x, coarse, xk, zk, B, varargin{:});
  gcp = g .* (cp - ck) + fk;
  gdcp = repmat(g, 1, length(x)) .* dcp;
  if w < 1.0
    % case: 0 < w < 1.0
    s = w * gcp + w1 * (D * (x - xk) + fk);
    ds = w * gdcp + w1 * D;
  else
    % case: w == 1.0 (use only the mapped coarse model)
    s = gcp;
    ds = gdcp;
  end
else
  % case: w == 0.0 (use only the linear fine model approximation)
  s = D * (x - xk) + fk;
  ds = D;
end
