function [s, ds] = combined_model(x, coarse, xk, zk, fk, B, D, w, varargin)
% A combined_model for the space mapping method
%
%   s_k(x) = w * c(p_k(x)) + (1 - w) * l_k(x)
%
% where c(p_k(x)) is a 'mapped_model'
% and l_k(x) = D_k * (x - x_k) + f(x_k) is a
% a linear approximation to f
%
% [function used in space mapping algorithms]

% js@imm.dtu.dk, January 19, 2003

w1 = 1 - w;

if w > 0.0
  [cp, dcp] = mapped_model(x, coarse, xk, zk, B, varargin{:});
  if w < 1.0
    % case: 0 < w < 1.0
    s = w * cp + w1 * (D * (x - xk) + fk);
    ds = w * dcp + w1 * D;
  else
    % case: w == 1.0 (use only the mapped coarse model)
    s = cp;
    ds = dcp;
  end
else
  % case: w == 0.0 (use only the linear fine model approximation)
  s = D * (x - xk) + fk;
  ds = D;
end
