function varargout = map(varargin)

% transforms the mapping parameters A, b and alpha
% to the mapping parameter vector p
% or
% transforms the mapping parameter vector p to the mapping parameters
% matrix A, vector b and scalar alpha
% 
% Input/output
% A     : (n,n) input mapping parameter matrix 
% b     : (n,1) input mapping parameter vector
% alpha : scalar output mapping parameter
% p     : mapping parameter vector p = [A(:); b; alpha]
% diag_a : determines the number of mapping parameters
%         0 : a full input mapping parameter matrix A is used (n*n + n + 1 mapping parameters)
%         1 : a diagonal input mapping parameter matrix A is used (2n + 1 mapping parameters)
% n     : size(A,1) = size(A,2) = length(b)
%
% Pernille Brock, IMM, DTU, September 7, 2004

if nargout == 1 & nargin == 4
    varargout{1} = Abalpha2p(varargin{:});
elseif nargout == 3 & nargin == 3
    [varargout{1:3}] = p2Abalpha(varargin{:});
else
    error('Wrong number of output arguments')
end

function p = Abalpha2p(A, b, alpha, diag_a)

if diag_a == 1 
    p = [diag(A); b; alpha];
elseif diag_a == 0
    p = [A(:); b; alpha];
else
    error('diag_a must have the value either 0 or 1')
end

function [A, b, alpha] = p2Abalpha(p, n, diag_a)

if diag_a == 1
    A = diag(p(1:n));
    b = p(n+1:2*n);
    alpha = p(end);
elseif diag_a == 0
    A = vec2mat(p(1:n*n),n)';
    b = p(n*n+1:end-1);
    alpha = p(end);
else
    error('diag_a must have the value either 0 or 1')
end