function [x, f, Hf] = smdirect(H, fun, x0, A, b, eq, opts, varargin)
%Direct, classical optimization.
%
%   [x, f, Hf] = smdirect(H, fun, x0, A, b, eq, opts, P1, P2, ...)
%

% js@imm.dtu.dk, January 26, 2003

switch H
case 'minimax'
  if exist('mincin')~=3
    % minimax, Matlabs optimization toolbox
    if opts.display == 0
      display = 'off';
    else
      display = 'iter';
    end
    [A, b, Aeq, beq] = split_constraints(A, b, eq);
    [x, f, Hf] = fminimax(fun, x0, A, b, Aeq, beq, [], [], [], optimset('Display', display, 'TolX',opts.epsilon,'TolFun',opts.epsilon), varargin{:});
  elseif 1
    % mincin, linearly constrained minimax, by Kaj Madsen and Jrgen Hald, IMM, DTU
    % - requires the Jacobian from fun
    f = feval(fun, x0, varargin{:});  % evaluate the model in order to determine the number of response functions
    [x, Hf, dx, maxfun, icontr, f, df, w] = mincin(@mincinobj, length(f), eq, b, -A, x0, {fun varargin{:}}, opts.dx, opts.epsilon, opts.kmax, 1);
  else
    % mincinf, linearly constrained minimax, by Jacob Sndergaard, IMM, DTU
    % - requires the Jacobian from fun
    % - depends on linprog from Matlabs optimization toolbox
    [x, f, df, k, stop] = mincinf(fun, x0, -A, b, eq, opts.dx, opts.epsilon, opts.kmax, varargin{:});
    Hf = max(f);
  end
case 2
  % least-squares
  if opts.display == 0
    display = 'off';
  else
    display = 'iter';
  end
  if ~isempty(b)
    % unconstrained minimization, Matlabs optimization toolbox
    % use the constraint optimization algorithm on the least-squares objective function
    [A, b, Aeq, beq] = split_constraints(A, b, eq);
    x = fmincon(@obj2, x0, A, b, Aeq, beq, [], [], [], optimset('LargeScale','off','Display',display,'TolX',opts.epsilon,'TolFun',opts.epsilon, 'GradObj', 'on'), fun, varargin{:});
    f = feval(fun, x, varargin{:});
  else
    if exist('marquardt') ~= 2
      % nonlinear least-squares, Matlabs optimization toolbox
      [x, Hf2, f] = lsqnonlin(fun, x0, [], [], optimset('Display',display,'TolX',opts.epsilon,'TolFun',opts.epsilon,'Jacobian','on'), varargin{:});
    else
      % Levenberg-Marquardt algorithm, by Hans Bruun Nielsen, IMM, DTU
      %[x, f] = marquardt(fun, x0, [10 max(opts.epsilon, 1e-8) opts.epsilon opts.kmax], varargin{:});
      [x, f] = marquardt(fun, x0, [1e-3 opts.epsilon_g opts.epsilon 100], varargin{:});     % modified by Pernille Brock, Sept. 2004
    end
  end
  Hf = smmerit(H, f);
otherwise
  if isnumeric(H), s = num2str(H); else s = H; end
  error(sprintf('direct optimization for the merit function "%s" is not implemented', s))
end

function [F, dF] = obj2(x, fun, varargin)
% least-squares objective function
[f, df] = feval(fun, x, varargin{:});
F = .5 * f'*f;
dF = df' * f;

function [f, df] = mincinobj(x, opt)
% mincin objective function
fun = opt{1};
if length(opt) > 1
  varargin = {opt{2:end}};
else
  varargin = {};
end
[f, df] = feval(fun, x, varargin{:});
