function [f, df] = rlcc(x, t)
% RLC parallel resonator (coarse model)

% js@imm.dtu.dk, January 9, 2003

if nargout > 1
  [f, df] = fdgrad(@RLC, x, [], 0, t.state);
else
  f = feval(@RLC, x, t.state);
end

function S11 = RLC(Xc, freq)
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Jose E. Rayas-Sanchez                                       March 29, 2000
% Simulation Optimization Systems Research Laboratory    McMaster University
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%                    RLC parallel resonator (coarse model)
% This function calculates S11 for a parallel RLC lumped resonator.
% A reference impedance of 50 ohms is used. It returns the magnitude of S11.
% Usage: S11 = RLC_c(Xc, freq)
%        Xc = [R L C]
%        R in ohms, L in nH, and C in pF
%        f in GHz (can be a vector of frequencies)
%        S11 is a vector containing the magnitude of S11 at the specified
%        frequencies in freq. The length of S11 = 2*length(freq).

Zo = 50;  % reference impedance for the S parameters
w = (2 * pi * 1e9) * freq;
R = real(Xc(1));
L = real(Xc(2));
C = real(Xc(3));
Yin = 1 / R + 1 ./ ((j * 1e-9 * L) * w) + (j * 1e-12 * C) * w;
Zin = 1 ./ Yin;
S11cx = (Zin - Zo) ./ (Zin + Zo);
S11 = abs(S11cx);