function [f, df] = rlcf1(x, t)
% RLC parallel resonator (fine model 1)

% js@imm.dtu.dk, January 9, 2003

if nargout > 1
  [f, df] = fdgrad(@RLC, x, [], 0, t.state);
else
  f = feval(@RLC, x, t.state);
end

function S11 = RLC(Xf, freq)
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Jose E. Rayas-Sanchez                                       March 29, 2000
% Simulation Optimization Systems Research Laboratory    McMaster University
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%                    RLC parallel resonator (fine model 1)
% This function calculates S11 for a parallel RLC lumped resonator with some
% parasitic elements. It has a exact linear mapping wrt RLC_c, valid everywhere.  
% A reference impedance of 50 ohms is used. It returns the magnitude of S11.
% Usage: S11 = RLC_f1(Xf, freq)
%        Xf = [R L C]
%        R in ohms, L in nH, and C in pF
%        f in GHz (can be a vector of frequencies)
%        S11 is a vector containing the magnitude of S11 at the specified
%        frequencies in freq. The length of S11 = 2*length(freq).

Zo = 50;  % reference impedance for the S parameters
w = (2 * pi * 1e9 ) * freq;
R = real(Xf(1));
L = real(Xf(2));
C = real(Xf(3));
Req = 5 + R;
Ceq = C + 6;
Yin = 1 / Req + 1 ./ ((j * 1e-9 * L) * w) + (j * 1e-12 * Ceq) * w;
Zin = 1 ./ Yin;
S11cx = (Zin - Zo) ./ (Zin + Zo);
S11 = abs(S11cx);