function [f, J] = rosenc(x, t)
% 'Coarse' Rosenbrock function
%
%    [f, J] = rosenc(x, t)
%
% t is a struct with elements A and b
%
% t.A : default eye(2)
% t.b : default [0 0]'
%
% same result can be obtained by the call
%    [f, J] = rosenf(t.A * x + t.b)
%
% hence the mapping is
%
%    p(x) = inv(t.A) * (x - t.b)

% js@imm.dtu.dk, January 9, 2003

if nargin < 2
  t = struct('A', eye(2), 'b', [0 0]');
else
  if ~isfield(t, 'A'), t.A = eye(2); end
  if ~isfield(t, 'b'), t.b = [0 0]'; end
end

if nargout > 1
  [f, J] = rosenf(t.A * x + t.b);
  J = J * t.A;
else
  f = rosenf(t.A * x + t.b);
end