function [f, df] = tlt7c(x, t)
% Seven-Section Impedance Transformer (coarse model)

% js@imm.dtu.dk, January 13, 2003

if nargout > 1
  [f, df] = fdgrad(@ZTRANc, x, [], 0, t.state);
else
    f = feval(@ZTRANc, x, t.state);
end

function mS11 = ZTRANc(L, freq);
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Ahmed S. Mohamed                                                 May, 2001
% Simulation Optimization Systems Research Laboratory    McMaster University
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%          Seven-Section Impedance Transformer (coarse model)
%
% This function calculates the magnitude of the reflection coefficient of a
% two-section impedance transformer with ideal transmission lines.
% Usage: mS11 = ZTRAN7c(L,freq)
%        L:    Vector of lengths of the transmission lines. L = [Li]',
%              i = 1,...,7, in degrees (electrical lengths).
%        freq: Vector of frequencies in GHz.
%        mS11: Vector of responses (magnitude of S11) at each frequency. It
%              is a row vector.
%

% optimal parameters L = [90 90 90 90 90 90 90]'
% for freq = [1:.1:7.7]';

Z = [91.94454481062100  85.52387852113699  78.15262602058000  70.71067811865500 63.97737676381300  58.46320450450900  54.38060529092400]; % Vector of charactteristic impedance

C = 3.0e8; % Speed of light in free-space.
Fo = 4.35e9; % Reference frequency for electrical length.
L = (C/(360*Fo))*L; % Converting electrical length to physical length.
freq = freq * 1e9;
RL = 100;
Ro = 50;

Omega = (2 * pi) * freq;     % angular frequencies
Lambda = C ./ freq;          % wave length
Beta = (2 * pi) ./ Lambda;   % propagation constant
factor = j * tan(Beta * L'); % factor matrix
Zin = RL;
for i = 1 : length(L)
  Zin = Z(i) * ((Zin + Z(i) * factor(:,i)) ./ (Z(i) + Zin .* factor(:,i))); % Get new input impedance
end
S11 = (Zin - Ro) ./ (Zin + Ro); % input impedance
mS11 = abs(S11); % store the aboslute value of the s11
